package sjl;

/**
 * <code>ArrayIterator</code> creates a iterator abstraction around the 
 * builtin array type. 
 * When using ArrayIterator, a java array can be adapted so it can 
 * be used as a container in the same way as all other containers.
 * Array iterators are created with the <code>Array.begin(Object[])</code> and 
 * <code>Array.end(Object[])</code> methods. <br>
 * Example:
 * <pre>
 *	Employee array[10];
 *	Algo.fill(Array.begin(array), Array.end(array), new Employee());
 * </pre>
 * which will insert a default employee object into all 10 positions of array.
 *
 * <p>Copyright <a href=..\Copyrght.htm>&copy</a> 1996 Finn Bock<p>
 */
class ArrayIterator extends Iter implements RandomIterator {
    Object array[];
    int pos;

    public ArrayIterator(Object arr[], int n) {
	array = arr;
	pos = n;
    }

    public int distance(Iterator i) {
	return ((ArrayIterator)i).pos - pos;
    }

    public boolean cmp(Iterator i) {
	return pos == ((ArrayIterator)i).pos;
    }

    public boolean less(Iterator i) {
	return pos < ((VectorIterator)i).pos;
    }

    public ArrayIterator copy() {
	return new ArrayIterator(array, pos);
    }

    public Iterator genericCopy() {
	return new ArrayIterator(array, pos);
    }

    public void setTo(Iterator i) {
        ArrayIterator ii = (ArrayIterator)i;
	array = ii.array;
	pos = ii.pos;
    }

    public Object get() {
	return array[pos];
    }

    public Object get(int n)  {
	return array[pos + n];
    }

    public Object put(Object o)  {
	return array[pos] = o;
    }

    public Object put(int n, Object o)  {
	return array[pos + n] = o;
    }

    public Iterator next() {
	pos++;
	return this;
    }

    public Iterator prev() {
	pos--;
	return this;
    }

    public RandomIterator prev(int n)  {
	pos -= n;
	return this;
    }

    public RandomIterator next(int n)  {
	pos += n;
	return this;
    }
}
