package sjl;

/**
 * Defines the methods allowed on associative container types. 
 * The supported containers are currently:
 * <code>Map</code>, <code>MultiMap</code>, <code>Set</code> and 
 * <code>MultiSet</code>.
 *
 * <p>Copyright <a href=..\Copyrght.htm>&copy</a> 1996 Finn Bock<p>
 *
 * @see Map
 * @see MultiMap
 * @see Set
 * @see MultiSet
 */
public interface AssociativeContainer extends Container {

    /**
     * Insert copies of <code>[first, last)</code>.
     */
    public abstract void insert(InputIterator first, InputIterator last);

    /**
     * Erase the element with a key equal to the object.
     */
    public abstract int erase(Object object);

    /**
     * Erase the elements in the range <code>[first,last)</code>.
     */
    public abstract void erase(Iterator first, Iterator last);

    /**
     * Return the iterator pointing to an element with a key equal to
     * key, or <code>end()</code> if such an element is not found.
     */
    public abstract ForwardIterator find(Object key);

    /**
     * Return the number of elements with key equal to
     * <code>key</code>.
     */
    public abstract int count(Object key);

    /**
     * Return iterators i and j such that all elements with keys equal
     * to <code>key</code> are in the range <code>[i,j)</code>.
     * The range is empty if no elements have the key.
     */
    public abstract Pair equal_range(Object key);
}
