package sjl;

/**
 * A predicate adapter. The <code>Bind1stPredicate</code> class can wrap a <code>Predicate2</code>
 * class so that it appears as a <code>Predicate1</code>.
 * <pre>
 *
 *     Algo.find(list.begin(), list.end(), 
 *               new Bind1stPredicate(new PredLessInteger(), new Integer(5)));
 * </pre>
 * The above example will return a iterator to the element in the list
 * with a value less than 5.
 *
 * <p>Copyright <a href=..\Copyrght.htm>&copy</a> 1996 Finn Bock<p>
 */
public class Bind1stPredicate implements Predicate1 {
    Predicate2 predicate;
    Object value;

    /**
     * Specified the predicate and the value to use.
     */
    public Bind1stPredicate(Predicate2 pred, Object val) {
	predicate = pred;
	value = val;
    }

    /**
     * Call the specified predicate using value and the argument.
     */
    public boolean compare(Object o)  {
	return predicate.compare(value, o);
    }
}
