package sjl;

/**
 * A function adapter. The <code>Bind2ndFunction</code> class can wrap 
 * a <code>Function2</code> class so that it appears as a 
 * <code>Function1</code>.
 * <pre>
 *
 *     Algo.transform(list.begin(), list.end(), list.begin()
 *               new Bind2ndFunction(new Integer(5), new FuncDifferenceInteger()));
 * </pre>
 * The above example will subtract the value of each integer element from 5
 * and store result in the list.
 *
 * <p>Copyright <a href=..\Copyrght.htm>&copy</a> 1996 Finn Bock<p>
 */
public class Bind2ndFunction implements Function1 {
    Function2 function;
    Object value;

    /**
     * Specified the function and the value to use.
     */
    public Bind2ndFunction(Function2 func, Object val) {
	function = func;
	value = val;
    }

    /**
     * Call the specified function using value and the argument.
     */
    public Object perform(Object o)  {
	return function.perform(o, value);
    }
}
