package sjl;

/**
 * A predicate adapter. The <code>Bind2ndPredicate</code> class can wrap 
 * a <code>Predicate2</code> class so that it appears as 
 * a <code>Predicate1</code>.
 * <pre>
 *
 *     Algo.find(list.begin(), list.end(), 
 *               new Bind2ndPredicate(new Integer(5), new FuncLessInteger()));
 * </pre>
 * The above example will return a iterator to the first element in the list
 * with a value greater or equal to 5.
 *
 * <p>Copyright <a href=..\Copyrght.htm>&copy</a> 1996 Finn Bock<p>
 */
public class Bind2ndPredicate implements Predicate1 {
    Predicate2 predicate;
    Object value;

    /**
     * Specified the predicate and the value to use.
     */
    public Bind2ndPredicate(Predicate2 pred, Object val) {
	predicate = pred;
	value = val;
    }

    /**
     * Call the specified predicate using value and the argument.
     */
    public boolean compare(Object o)  {
	return predicate.compare(o, value);
    }
}
