package sjl;

/**
 * Defines the methods allowed on all container types. This is the absolute 
 * minimum set of operations that all container types must support.
 *
 * <p>Copyright <a href=..\Copyrght.htm>&copy</a> 1996 Finn Bock<p>
 *
 * @see List
 * @see Vector
 * @see Deque
 * @see Array
 * @see Set
 * @see MultiSet
 * @see Map
 * @see MultiMap
 */
public interface Container {

    /** 
     * Erase all elements in the container.
     */
    public abstract void flush();

    /**
     * Compare the elements in this container with the elements 
     * in another container.
     * @return <code>true</code> is the elements match.
     */
    public abstract boolean equals(Object container);

    /**
     * Return an iterator that points to the beginning of the container. 
     */
    public abstract ForwardIterator beginGeneric();

    /**
     * Return an iterator that points past the end of the container. 
     */
    public abstract ForwardIterator endGeneric();

    /**
     * Return the number of element that the container contains.
     */
    public abstract int size();

    /**
     * Return <code>true</code> of the container does not contain any elements.
     */
    public abstract boolean empty();

    /**
     * Return the size of the largets possible container.
     */
    public abstract int max_size();
}
