package sjl;

class DequeIterator extends Iter implements RandomIterator {
    Deque deque;
    protected int current;
    protected int node;

    public int distance(Iterator i) {
	DequeIterator ii = (DequeIterator)i;
	return node == ii.node ? ii.current - current :
		deque.buffer_size * (ii.node - node - 1) + 
		ii.current + (deque.buffer_size - current);
    }

    public DequeIterator(DequeIterator i) {
	deque = i.deque;
	current = i.current;
	node = i.node;
    }

    public DequeIterator(Deque q, int current, int node) {
	deque = q;
	this.node = node;
	this.current = current;
    }

    public DequeIterator copy() {
	return new DequeIterator(deque, current, node);
    }

    public Iterator genericCopy() {
	return new DequeIterator(deque, current, node);
    }

    public void setTo(Iterator i) {
        DequeIterator ii = (DequeIterator)i;
	node = ii.node;
	current = ii.current;
    }

    protected void _set(int current, int node) {
        this.node = node;
	this.current = current;
    }

    /**
     * sjl.Iterator stuff
     */
    public boolean cmp(Iterator i)  {
        DequeIterator ii = (DequeIterator)i;
	return current == ii.current ||
		((current == 0 || ii.current == 0) && distance(ii) == 0);
    }

    public boolean less(Iterator i) {
        DequeIterator ii = (DequeIterator)i;
	return (node == ii.node) ? (current < ii.current) : (node < ii.node);
    }

    /**
     * sjl.RandomIterator stuff
     */
    public RandomIterator next(int n)  {
	int offset = n + current;
	int jump = offset >= 0
		? offset / deque.buffer_size
		: -((-offset + deque.buffer_size - 1) / deque.buffer_size);
	if (jump == 0)
	    current += n;
	else {
	    node += jump;
	    current = offset - jump * deque.buffer_size;
	}
	return this;
    }

    /**
     * sjl.ForwardIterator stuff
     */
    public Iterator next()  {
	if (++current == deque.buffer_size) {
	    ++node;
	    current = 0;
	}
	return this;
    }

    /**
     * sjl.RandomIterator stuff
     */
    public RandomIterator prev(int n)  {
	next(-n);
	return this;
    }

    /**
     * sjl.BidirectionalIterator stuff
     */
    public Iterator prev()  {
	if (current == 0) {
	    --node;
	    current = deque.buffer_size;
	}
	--current;
	return this;
    }

    /**
     * sjl.RandomIterator stuff
     */
    public Object get(int n)  {
	next(n);
	Object o = deque.map[node][current];
	prev(n);
	return o;
    }

    /**
     * sjl.InputIterator stuff
     */
    public Object get()  {
	return deque.map[node][current];
    }

    /**
     * sjl.RandomIterator stuff
     */
    public Object put(int n, Object o)  {
	next(n);
	deque.map[node][current] = o;
	prev(n);
	return o;
    }

    /**
     * sjl.OutputIterator stuff
     */
    public Object put(Object o)  {
//System.out.println(deque);
//System.out.println(deque.map);
//System.out.println(deque.map[node]);
//System.out.println(node);
//System.out.println(current);
	return deque.map[node][current] = o;
    }

   public String toString()  {
        StringBuffer buf = new StringBuffer();
        buf.append("deque[" + node + "," + current + "]");
        return buf.toString();
    }
}
