package sjl;

/**
 * Defines the interface for function object that does not take any parameters.
 * A function object is an object that can understand the 
 * <code>perform()</code> message. 
 * Function objects are used by many of the algorithms, 
 * ex: <a href=sjl.Algo.html#generate>generate</a>.
 * You must implement the actual function object with code like this:
 * <pre>
 * public class GenerateString implements Function0 {
 *     private cnt = 1;
 *     public Object perform() {
 *         return new String("Initial value : " + n++);
 *     }
 * }
 * </pre>
 * This class will, when used with the <code>generate</code> function, fill 
 * the container with the strings:
 * <pre>
 *      Initial value : 1
 *      Initial value : 2
 *      Initial value : 3
 *      ...
 * </pre>
 *
 * <p>Copyright <a href=..\Copyrght.htm>&copy</a> 1996 Finn Bock<p>
 *
 * @see Function1
 * @see Function2
 */
public interface Function0 {

    /**
     * This method is executed for each call to the function object.
     */
    public abstract Object perform();
}
