package sjl;

/**
 * Defines the interface for function object that takes one parameter.
 * A function object is an object that can understand the 
 * <code>perform(Object)</code> message. 
 * Function objects are used by many of the algorithms, 
 * ex: <a href=sjl.Algo.html#for_each>for_each</a>.
 * You must implement the actual function object with code like this:
 * <pre>
 * public class Print implements Function1 {
 *     public Object perform(Object each) {
 *         System.out.println(each.toString());
 *         return null;
 *     }
 * }
 * </pre>
 * This class will, when used with the <code>for_each</code> function 
 * print each element of the container.
 *
 * <p>Copyright <a href=..\Copyrght.htm>&copy</a> 1996 Finn Bock<p>
 *
 * @see Function0
 * @see Function2
 */
public interface Function1 {

    /**
     * This method is executed for each call to the function object.
     */
    public abstract Object perform(Object o);
}
