package sjl;

/**
 * Defines the interface for functions object that takes two parameters.
 * A function object is an object that can understand the 
 * <code>perform(Object, Object)</code> message. 
 * Function objects are used by many of the algorithms, 
 * ex: <a href=sjl.Algo.html#accumulate>accumulate</a>.
 * You must implement the actual function object with code like this:
 * <pre>
 * public class Sum implements Function2 {
 *     public Object perform(Object o1, Object o2) {
 *         return new Integer(((Integer)o1).intValue() + ((Integer)o2).intValue());
 *     }
 * }
 * </pre>
 * This class will, when used with the <code>accumulate</code> function 
 * add together the integer value of each corresponding element in the 
 * two containers. The above example already exist as 
 * <code>sjl.wrappers.FuncSumInteger</code>.
 *
 * <p>Copyright <a href=..\Copyrght.htm>&copy</a> 1996 Finn Bock<p>
 *
 * @see Function0
 * @see Function1
 */
public interface Function2 {

    /**
     * This method is executed for each call to the function object.
     */
    public abstract Object perform(Object o1, Object o2);
}
