package sjl;

/**
 * <code>HashMultiSet</code> is a kind of associative container that supports 
 * unique keys (contains at most one of each key value) and provides 
 * for fast retrieval of the keys themselves.
 *
 * <p>Copyright <a href=..\Copyrght.htm>&copy</a> 1996 Finn Bock<p>
 */

public class HashMultiSet implements AssociativeContainer {
    Hashtbl ht;
    HashtblIterator end;
    HashtblIterator begin;
    HashtblIterator tmpiterator;

    /**
     * Construct an empty set. 
     */
    public HashMultiSet() {
	ht = new Hashtbl(1009, new HashFunction(), new PredEqual(), new Ident(), true);
    }

    /**
     * Construct a set and initialize it with the contents of another set. 
     * @param set Initialize the new set with the elements of set.
     */
/*
    public HashMultiSet(HashMultiSet set) {
	ht = new Hashtbl(set.ht, true);
    }
*/

    /**
     * Construct a set and initialize it with the elements in the 
     * range <code>[first,last)</code>.
     * @param first The beginning of the range.
     * @param last The end of the range.
     */
    public HashMultiSet(InputIterator first, InputIterator last) {
	this();
	for (InputIterator i = Iter.copy(first); !i.cmp(last); i.next())
	    ht.insert(i.get());
    }

    /**
     * Erase all the elements in the container. 
     */
    public void flush() {
//	this();
    }

    /**
     * Compare the elements in this container with the elements 
     * in another container.
     * @return <code>true</code> is the elements match.
     */
    public boolean equals(Object container) {
	if ((container != null) && (container instanceof Container)) {
	    Container c = (Container)container;
	    return size() == c.size() &&  Algo.equal(beginRef(), endRef(),
			(InputIterator)c.beginGeneric());
	}
	return false;
    }

    /**
     * Returns the iterator that represents the beginning of the set. 
     */
    public ForwardIterator begin() {
	return ht.begin();
    }

    /**
     * Returns a reference to the iterator that represents the beginning 
     * of the set. 
     */
    public ForwardIterator beginRef() {
	return ht.begin();
    }

    /**
     * Returns the iterator that represents the end of the set. 
     */
    public ForwardIterator end() {
	return ht.end();
    }

    /**
     * Returns a reference to the iterator that represents the end of the set. 
     */
    public ForwardIterator endRef() {
	return ht.end();
    }

    /**
     * Returns the iterator that represents the beginning of the set as an
     * untyped iterator. 
     */
    public ForwardIterator beginGeneric() {
	return ht.begin();
    }

    /**
     * Returns the iterator that represents the end of the set as an
     * untyped iterator. 
     */
    public ForwardIterator endGeneric()  {
	return ht.end();
    }

    /**
     * Returns the number of elements in the set.
     */
    public int size() {
	return ht.size();
    }

    /**
     * Returns the maximum number of elements allowed in a set.
     */
    public int max_size() {
	return Integer.MAX_VALUE;
    }

    /**
     * Returns <code>true</code> if the set does not contain any elements.
     */
    public boolean empty() {
	return ht.size() == 0;
    }

    /**
     * Insert an element into the set. The element is only inserted if 
     * there is no element in the container with a key equal to the key 
     * of value.
     * @param value The element to insert.
     * @return An iterator that point to the newly inserted element
     */
    public HashtblIterator insert(Object value) {
	return (HashtblIterator)ht.insert(value).first;
    }

    /**
     * Insert the elements from the range <code>[first,last)</code> into 
     * the set.
     * @param first The beginning of the range.
     * @param last The end of the range.
     */
    public void insert(InputIterator first, InputIterator last) {
	for (InputIterator i = Iter.copy(first); !i.cmp(last); i.next()) 
	    ht.insert(i.get());
    }

    /**
     * Erase the element pointed to by the iterator.
     * @param position Points to the element to be removed.
     */
    public void erase(Iterator position) {
	ht.erase((HashtblIterator)position);
    }

    /**
     * Erase the element with key equal to value.
     * @return The number of element erased.
     */
    public int erase(Object value) {
	return ht.erase(value);
    }

    /**
     * Erase the elements in the range <code>[first,last)</code>.
     * @param first The beginning of the range.
     * @param last The end of the range.
     */
    public void erase(Iterator first, Iterator last) {
	ht.erase((HashtblIterator)first, (HashtblIterator)last);
    }

    /**
     * Search the set for an element with a key equal to value.
     * @param value The key to search for.
     * @return An Iterator pointing to the element with a key equal to value,
     *          or end() if such an element is not found.
     */
    public ForwardIterator find(Object value) {
	return ht.find(value);
    }

    /**
     * Returns the number of element with a key equal to value.
     */
    public int count(Object value) {
	return ht.count(value);
    }


    /**
     * Returns a pair like 
     * <code>Pair(lower_bound(value), upper_bound(value))</code>.
     */
    public Pair equal_range(Object value) {
	return ht.equal_range(value);
    }

    private void dump() {
	ht.dump();
    }
}
