package sjl;

class Hashtbl {
    boolean insertAlways;	// True if dups should be inserted.
    int elementCount;		// # of actual elements.
    int totBuckets;		// # of buckets allocated.
    int usedBuckets;		// # of buckets actually being used.
    float maxLF;		// LF that triggers expansion.
    float minLF;		// LF that triggers contraction.
    Deque buckets;		// Where the elements are actually stored.
    int firstNeb;		// first non-empty bucket, -1 if empty.
    int lastNebIns;		// last non-empty bucket, -1 if empty.

    HashFunction hf;
    Predicate2 eql;
    protected Function1 keyvalue;

    public Hashtbl(int table_size, HashFunction hashfunc, Predicate2 equal, Function1 keyvalue, boolean always) {
	init(table_size, always);
	hf = hashfunc;
	eql = equal;
	this.keyvalue = keyvalue;
    }

    public HashtblIterator begin() {
	if (!empty())
	    return new HashtblIterator(this, getBucket(firstNeb),
			getBucket(firstNeb).begin());
	else
	    return end();
    }

    public HashtblIterator end() {
	return new HashtblIterator(this, null, null);
    }

    public int size() {
	return elementCount;
    }

    public boolean empty() {
	return size() == 0;
    }

    private int bucketCount() {
	return usedBuckets;
    }


    private float loadFactor() {
	return (size() == 0 ? 0 : (float)size() / bucketCount());
    }

    public void erase(HashtblIterator position) {
	int hash_value = S(keyvalue.perform(position.ptr));
	HashtblBucket b = getBucket(hash_value);
	b.erase(position.ptr);
	--elementCount;
	if (loadFactor() < minLF)
	    contract();
	else if (b.empty())
	    eraseNeb(b, hash_value);
    }

    public int erase(Object key) {
	Pair p = equal_range(key);
	int n = distance((HashtblIterator)p.first, (HashtblIterator)p.second);
	erase((HashtblIterator)p.first, (HashtblIterator)p.second);
	return n;
    }

    public void erase(ForwardIterator first, ForwardIterator last) {
	first = Iter.copy(first);
	while (!first.cmp(last)) {
	    erase(first);
	    first.next();
	}
    }

    public void init(int table_size, boolean insert_always) {
	elementCount = 0;
	totBuckets = table_size;
	usedBuckets = table_size;
	maxLF = (float)2.0;
	minLF = (float)0.5;
	buckets = new Deque(table_size, null);
//	for (int i = 0; i < table_size; i++)
//	    buckets.push_back(new HashtblBucket());
	firstNeb = -1;
	lastNebIns = -1;
	insertAlways = insert_always;
    }

    public void dump() {
	System.out.println("");
    }


    public Pair equal_range(Object key) {
	ForwardIterator p = find(key);
	ForwardIterator q = Iter.copy(p);
	while (!p.cmp(end()) && eql.compare(keyvalue.perform(p.get()), key)) {
	    p.next();
	}
	    
	return new Pair(q, p);
    }

    public int count(Object value) {
	Pair p = equal_range(value);
	return distance((HashtblIterator)p.first, (HashtblIterator)p.second); 
    }

    public HashtblIterator find(Object key) {
	int hash_value = S(key);
	HashtblBucket b = getBucket(hash_value);
	for (ForwardIterator p = b.begin(); !p.cmp(b.endRef()); p.next()) {
	    if (eql.compare(keyvalue.perform(p.get()), key))
		return new HashtblIterator(this, b, p);
	}
	return end();
    }

    public int distance(ForwardIterator first, ForwardIterator last) {
	int n = 0;
	first = Iter.copy(first);
	while (!first.cmp(last)) {
	    first.next();
	    n++;
	}
	return n;
    }

    public Pair insert(Object value) {
	int hash_value = S(keyvalue.perform(value));
	HashtblBucket b = getBucket(hash_value);
	ForwardIterator p;
	for (p = b.begin(); !p.cmp(b.end()); p.next()) 
	    if (eql.compare(keyvalue.perform(p.get()), keyvalue.perform(value)))
		break;
	ForwardIterator where = Iter.copy(p);
	boolean inserted = false;

	if (b.empty()) 
	    insertNeb(b, hash_value);
	if (p.cmp(b.end()) || insertAlways) {
	    if (p.cmp(b.end())) {
		b.push_front(value);
		where = b.begin();
	    } else {
		p.next();
		where = (ForwardIterator)b.insert(p, value);
	    }
	    inserted = true;
	    ++elementCount;
	    if (loadFactor() > maxLF)
		expand();
	}
	return new Pair(new HashtblIterator(this, b, where), new Boolean(inserted));
    }

    public void insert(InputIterator first, InputIterator last) {
	first = Iter.copy(first);
	while (!first.cmp(last)) {
	    insert(first.get());
	    first.next();
	}
    }

    private Pair pair_iterator_bool(HashtblIterator x, boolean bool) {
	return new Pair(x, new Boolean(bool)); // XXX:
    }

    protected int S(Object key) {
	int hv = hf.hashCode(key) % totBuckets;
	if (hv < 0)
	    hv = -hv;
	if (hv >= usedBuckets)
	    return hv - (totBuckets >> 1);
	else
	    return hv;
    }

    private void contract() {
	--usedBuckets;
	merge(usedBuckets);
	if (usedBuckets < (totBuckets >> 1))
	    totBuckets /= 2;
    }

    private void expand() {
	++usedBuckets;
	if (usedBuckets > totBuckets)
	    totBuckets *= 2;
	if (usedBuckets > buckets.size())
	    buckets.push_back(new HashtblBucket());
	split(usedBuckets - 1);
    }

    private void merge(int btm) {
	int target = btm - (totBuckets >> 1);
	HashtblBucket bBtm = getBucket(btm);
	HashtblBucket bTarget = getBucket(target);
	if (!bBtm.empty()) {
	    bTarget.splice(bBtm);
	    eraseNeb(bBtm, btm);
	}
//	System.out.println("Merged bucket #:" + btm);
    }

    private void split(int target) {
	int bts = target - (totBuckets >> 1);
	HashtblBucket bBts = getBucket(bts);
	HashtblBucket bTarget = getBucket(target);
	ForwardIterator p = bBts.begin();

	if (!bBts.empty()) {
	    // Note placement of p++ in following code; this allows p to be
	    // properly advanced before node referred to by p is moved.
	    while (!p.cmp(bBts.endRef()))
		if (S(keyvalue.perform(p.get())) != bts) {
		    ForwardIterator pp = Iter.copy(p);
		    p.next();
		    bTarget.splice(bBts, pp);
		} else
		    p.next();
	}
//	System.out.println("Splice bucket #:" + bts);
	if (bBts.empty())
	    eraseNeb(bBts, bts);
	if (!bTarget.empty())
	    insertNeb(bTarget, target);
    }

    private void eraseNeb(HashtblBucket b, int bNumber) {
	if (b.prev != -1)
	    getBucket(b.prev).next = b.next;
	if (b.next != -1)
	    getBucket(b.next).prev = b.prev;
	if (bNumber == firstNeb)
	    firstNeb = b.next;
	if (bNumber == lastNebIns)
	    lastNebIns = (b.next != -1 ? b.prev : b.next);
	b.next = b.prev = -1;
    }

    private void insertNeb(HashtblBucket b, int bNumber) {
	if (firstNeb != -1) {  // The Hashtbl is not empty
	    getBucket(lastNebIns).next = bNumber;
	    b.prev = lastNebIns;
	} else
	    firstNeb = bNumber;
	lastNebIns = bNumber;
    }


    private HashtblBucket getBucket(int hash_value) {
	HashtblBucket b = (HashtblBucket)buckets.get(hash_value);
	if (b == null)
	    b = (HashtblBucket)buckets.put(hash_value, new HashtblBucket());
	return b;
    }
	
}

