package sjl;

class HashtblIterator extends Iter implements ForwardIterator {
    Hashtbl hashtbl;
    HashtblBucket hashbucket;
    ForwardIterator ptr;

    public int distance(Iterator end) {
	int n = 0;
	ForwardIterator i = Iter.copy(this);
	while (!i.cmp(end)) {
	    i.next();
	    ++n;
	}
	return n;
    }

    public HashtblIterator(HashtblIterator i) {
	hashtbl = i.hashtbl;
	hashbucket = i.hashbucket;
	ptr = (i.ptr == null) ? null : Iter.copy(i.ptr);
    }

    public HashtblIterator(Hashtbl v, HashtblBucket b, ForwardIterator n) {
	hashtbl = v;
	hashbucket = b;
	ptr = n;
    }

    public HashtblIterator copy() {
	return new HashtblIterator(hashtbl, hashbucket, 
		ptr == null ? null : Iter.copy(ptr));
    }

    public Iterator genericCopy() {
	return new HashtblIterator(hashtbl, hashbucket, 
		ptr == null ? null : Iter.copy(ptr));
    }

    public void setTo(Iterator i) {
        HashtblIterator ii = (HashtblIterator)i;
	hashtbl = ii.hashtbl;
	hashbucket = ii.hashbucket;
	ptr = ii.ptr;
    }

    public boolean cmp(Iterator i)  {
        HashtblIterator ii = (HashtblIterator)i;
	if (ptr == null || ii.ptr == null)
	    return ptr == ii.ptr;
	return ptr.cmp(ii.ptr);
    }

    public Iterator next()  {
	ptr.next();
	if (ptr.cmp(hashbucket.end())) {
	    if (hashbucket.next != -1) {
	        hashbucket = (HashtblBucket)hashtbl.buckets.get(hashbucket.next);
	        ptr = hashbucket.begin();
	    } else
		ptr = null;
	}
	return this;
    }

    public Object get()  {
	return ptr.get();
    }

    public Object put(Object value)  {
	hashtbl.insert(value);
	return value;
    }

    public String toString()  {
        StringBuffer buf = new StringBuffer();
        buf.append("HashtblIterator[");
        buf.append(ptr);
        buf.append("]");
        return buf.toString();
    }
}
