package sjl;

/**
 * Base class of all concrete iterators. It implements the common behavior 
 * that are shared between all concrete iterators.
 *
 * <p>Copyright <a href=..\Copyrght.htm>&copy</a> 1996 Finn Bock<p>
 */
public class Iter {

    /**
     * Return a new copy of the parameter.
     */
    public static InputIterator copy(InputIterator iter) {
	return (InputIterator)iter.genericCopy();
    }

    /**
     * Return a new copy of the parameter.
     */
    public static OutputIterator copy(OutputIterator iter) {
	return (OutputIterator)iter.genericCopy();
    }

    /**
     * Return a new copy of the parameter.
     */
    public static ForwardIterator copy(ForwardIterator iter) {
	return (ForwardIterator)iter.genericCopy();
    }

    /**
     * Return a new copy of the parameter.
     */
    public static BidirectionalIterator copy(BidirectionalIterator iter) {
	return (BidirectionalIterator)iter.genericCopy();
    }

    /**
     * Return a new copy of the parameter.
     */
    public static RandomIterator copy(RandomIterator iter) {
	return (RandomIterator)iter.genericCopy();
    }

    /**
     * A casting method. Return self as a InputIterator.
     */
    public InputIterator inputIterator() {
	return (InputIterator)this;
    }

    /**
     * A casting method. Return self as an OutputIterator, or fails with 
     * a runtime error message if this is not a output iterator.
     */
    public OutputIterator outputIterator() {
	return (OutputIterator)this;
    }

    /**
     * A casting method. Return self as an ForwardIterator, or fails with 
     * a runtime error message if this is not a forward iterator.
     */
    public ForwardIterator forwardIterator() {
	return (ForwardIterator)this;
    }

    /**
     * A casting method. Return self as an BidirectionaIterator, or fails 
     * with a runtime error message if this is not a bidirectional iterator.
     */
    public BidirectionalIterator bidirectionalIterator() {
	return (BidirectionalIterator)this;
    }

    /**
     * A casting method. Return self as an RandomIterator, or fails with 
     * a runtime error message if this is not a random iterator.
     */
    public RandomIterator randomIterator() {
	return (RandomIterator)this;
    }
}
