package sjl;

/**
 * The mininum interface that all iterators have in common. 
 * It servers as a generic type place holder while the iterator is being 
 * passed around.
 *
 * <p>Copyright <a href=..\Copyrght.htm>&copy</a> 1996 Finn Bock<p>
 *
 * @see InputIterator 
 * @see OutputIterator
 * @see ForwardIterator 
 * @see BidirectionalIterator 
 * @see RandomIterator
 * @see Iter
 */

public interface Iterator {

    /**
     * Advance self to the next element in the container.
     * @return self
     */
    public abstract Iterator next();

    /**
     * Compare self with another Iterator.
     * This method is implemented in the concrete iterator classes.
     * @param i the iterator to compare with.
     * @return true if the two iterators are equal (points to the same place).
     */
    public abstract boolean cmp(Iterator i);

    /**
     * Return a copy of myself. This copy can then be manipulated. 
     * @return a copy of myself
     */
    public abstract Iterator genericCopy();

}
