package sjl;

/**
 * A predicate adapter. The <code>Not1</code> class can wrap a 
 * <code>Predicate1</code> class so that the truth value is reversed.
 * <pre>
 *     public class IsGreen implements Predicate1 {
 *         public boolean compare(Object car) {
 *             return ((Car)car).color = "Green";
 *         }
 *     }
 *
 *     Algo.find(list.begin(), list.end(), new Not1(new IsGreen()));
 * </pre>
 * The above example will return a iterator to the first car that is 
 * not green.
 *
 * <p>Copyright <a href=..\Copyrght.htm>&copy</a> 1996 Finn Bock<p>
 */
public class Not1 implements Predicate1 {
    Predicate1 predicate;

    /**
     * Specified the predicate to use.
     */
    public Not1(Predicate1 pred) {
	predicate = pred;
    }

    /**
     * Negate the thruth value of the predicate. The predicate is 
     * specified when the adapter is constructed.
     */
    public boolean compare(Object o)  {
	return !predicate.compare(o);
    }
}
