package sjl;

/**
 * A predicate adapter. The <code>Not2</code> class can wrap 
 * a <code>Predicate2</code> class so that the truth value is reversed.
 * <pre>
 *     Algo.sort(vector.begin(), vector.end(), new Not2(new PredLessInteger()));
 * </pre>
 * The above example will sort a vector of Integers in reverse.
 *
 * <p>Copyright <a href=..\Copyrght.htm>&copy</a> 1996 Finn Bock<p>
 */
public class Not2 implements Predicate2 {
    Predicate2 predicate;

    /**
     * Specified the predicate to use.
     */
    public Not2(Predicate2 pred) {
	predicate = pred;
    }

    /**
     * Negate the thruth value of the predicate. The predicate is 
     * specified when the adapter is constructed.
     */
    public boolean compare(Object o1, Object o2)  {
	return !predicate.compare(o1, o2);
    }
}
