package sjl;
import java.io.*;

/**
 * To make it possible for the generic algorihms to work directly with
 * output streams, the <code>OutputStreamIterator</code> have been created. <br>
 * Ex:
 * <pre>
 *     fout = new FileOutputStream("outputfile");
 *     OutputStreamIterator is = new OutputStreamIterator(fout, new WriteIntegerText(), " ");
 *     Algo.copy(vector.begin(), vector.end(), is);
 * </pre>
 * The <code>OutputStreamIterator</code> writes (using the 
 * <a href=sjl.FuncTextWriter.html>FuncTextWriter</a> function object) 
 * successive elements onto the output stream (fout) for which it was 
 * contructed. 
 * The last argument, called a <i>delimiter string</i>, is written to 
 * the stream after each element is written.
 * Ever time the <code>put()</code> method is called the element is written 
 * to the stream.
 * <p>
 * It is not possibly to get a value out of the output iterator. 
 * Its only use is as an output iterator is situation like this
 * <pre>
 *     while (!first.cmp(last)) {
 *         result.put(first.get());
 *         result.next();
 *         first.next();
 *     }
 * </pre>
 *
 * <p>Copyright <a href=../Copyrght.htm>&copy</a> 1996 Finn Bock<p>
 */ 
public class OutputStreamIterator extends Iter implements OutputIterator {
    DataOutputStream stream;
    String string;
    Function2 writer;

    /** 
     * Construct an OutputStreamIterator for the stream using the function
     * object as a writer.
     * @param ostream The iterator is constructed for this stream. A 
     *                DataOutputStream is constructed for istream.
     * @param func The function object used as a writer.
     * @param string The delimiter string
     * @see DataOutputStream
     */
    public OutputStreamIterator(OutputStream ostream, Function2 func, String string) {
	init(new DataOutputStream(ostream), func, string);
    }

    /** 
     * Construct a OutputStreamIterator for the stream using the function
     * object as a writer.
     * @param ostream The iterator is constructed for this stream. 
     * @param string The delimiter string
     */
    public OutputStreamIterator(DataOutputStream ostream, Function2 func, String delim) {
	init(ostream, func, string);
    }

    /**
     * Advance the iterator to the next element. Does nothing.
     */
    public Iterator next()  {
	return this;
    }

    /**
     * Write a value to the output stream.
     */
    public Object put(Object value)  {
        writer.perform(stream, value);
        if (string != null) {
	    try {
	        stream.writeBytes(string);
	    } catch (IOException e) { }
	}
	return value;
    }

    /**
     * Compare two iterators.
     * @return false.
     */
    public boolean cmp(Iterator iterator)  {
	return false;
    }

    /**
     * XXX: Is this possible???
     * @return this
     */
    public Iterator genericCopy() {
	return this;
    }

    private void init(DataOutputStream istream, Function2 func, String delim) {
	stream = istream;
	writer = func;
	string = delim;
    }
}
