package sjl;

/**
 * Defines the interface for predicate objects that takes one parameter.
 * A predicate object is an object that can understand the 
 * <code>compare(Object)</code> message. 
 * It is a special case of function objects. 
 * A predicate's <code>compare()</code> always returns a boolean whereas 
 * a function object's <code>perform()</code> always return 
 * an <code>Object</code>. 
 * <br>
 * Predicates are typically used for by the Algorithms for searching,
 * ex: <a href=sjl.Algo.html#remove_copy>remove_copy</a>.
 * You must implement the actual function object with code like this:
 * <pre>
 *     public class IsMale implements Predicate1 {
 *         public boolean compare(Object person) {
 *             return ((Employee)person).sex = "M";
 *         }
 *     }
 * </pre>
 * This class will, when used with the <code>remove_copy</code> function 
 * insert all male colleagues into the result container.
 *
 * <p>Copyright <a href=..\Copyrght.htm>&copy</a> 1996 Finn Bock<p>
 *
 * @see Predicate2
 */
public interface Predicate1 {

    /**
     * This method is executed for each call to the function object.
     */
    public abstract boolean compare(Object o);
}
