package sjl;

/**
 * Defines the interface for predicate objects that takes two parameter.
 * A predicate object is an object that can understand the 
 * <code>compare(Object, Object) </code> message. 
 * It * is a special case of function objects. 
 * A predicate's <code>compare()</code> always returns a boolean 
 * whereas a function object's <code>perform()</code> return 
 * an <code>Object</code>. 
 * <br>
 * Predicates are typically used by the Algorithms for sorting, 
 * ex: <a href=sjl.Algo.html#sort>sort</a>.
 * You must implement the actual function object with code like this:
 * <pre>
 *     public class IsLess implements Predicate2 {
 *         public boolean compare(Object o1, Object o2) {
 *             return o1.equals(o2);
 *         }
 *     }
 * </pre>
 *
 * <p>Copyright <a href=..\Copyrght.htm>&copy</a> 1996 Finn Bock<p>
 *
 * @see Predicate1
 */
public interface Predicate2 {

    /**
     * This method is executed for each call to the function object.
     */
    public abstract boolean compare(Object o1, Object o2);
}
