package sjl;

/**
 * <code>PriorityQueue</code> is a container adapter where the element
 * immediatly available for retrival is the largest of those in the sequence.
 * The container that can be turned into a <code>PriorityQueue</code>
 * must implement the 
 * <a href=sjl.BackInsertContainer.html>BackInsertContainer</a>
 * interface. Currently <code>Vector</code> and <code>Deque</code>
 * can be used as containers for a <code>PriorityQueue</code>.
 *
 * <p>Copyright <a href=..\Copyrght.htm>&copy</a> 1996 Finn Bock<p>
 * 
 * @see Vector
 * @see Deque
 */ 

public class PriorityQueue {
    BackInsertContainer container;
    Predicate2 predicate;

    /**
     * Construct a new PriorityQueue based on the container argument.
     */
    public PriorityQueue(BackInsertContainer container, Predicate2 pred) {
	this.container = container;
	predicate = pred;
	Algo.make_heap((RandomIterator)container.beginGeneric(), 
		(RandomIterator)container.endGeneric(), predicate);
    }

    /**
     * Compare the elements in this container with the elements 
     * in another container.
     * @return <code>true</code> is the elements match.
     */
    public boolean equals(Object container) {
	if ((container != null) && (container instanceof Container)) {
	    Container c = (Container)container;
	    return this.container.size() == c.size() &&
		 Algo.equal((InputIterator)this.container.beginGeneric(), 
			(InputIterator)this.container.endGeneric(),
			(InputIterator)c.beginGeneric());
	}
	return false;
    }

    /**
     * Returns the number of elements stored in the vector.
     */
    public int size() {
	return container.size();
    }

    /**
     * Returns <code>true</code> if the queue does not contain any elements.
     */
    public boolean empty() {
	return container.empty();
    }

    /**
     * Returns top element on the queue.
     */
    public Object top() {
	return container.front();
    }

    /**
     * Removes the top element on the queue.
     */
    public void pop() {
	Algo.pop_heap((RandomIterator)container.beginGeneric(), 
		(RandomIterator)container.endGeneric(), predicate);
	container.pop_back();
    }

    /**
     * Insert a new element on the top of the PriorityQueue.
     */
    public void push(Object o) {
	container.push_back(o);
	Algo.push_heap((RandomIterator)container.beginGeneric(), 
		(RandomIterator)container.endGeneric(), predicate);
    }
}
