package sjl;

/**
 * <code>Queue</code> is a container adapter that provides a 
 * <i>first-in, first-out</i> protocol to the elements in the 
 * container. The container must implement the 
 * <a href=sjl.QueueContainer.html>QueueContainer</a> interface. 
 * The <code>List</code> and <code>Deque</code> containers implements 
 * the <code>QueueContainer</code> interface.
 *
 * <p>Copyright <a href=..\Copyrght.htm>&copy</a> 1996 Finn Bock<p>
 *
 * @see List
 * @see Deque
 */ 

public class Queue {
    QueueContainer container;

    /**
     * Construct a new Queue based on the container argument.
     */
    public Queue(QueueContainer container) {
	this.container = container;
    }

    /**
     * Compare the elements in this container with the elements 
     * in another container.
     * @return <code>true</code> is the elements match.
     */
    public boolean equals(Object container) {
	if ((container != null) && (container instanceof Container)) {
	    Container c = (Container)container;
	    return this.container.size() == c.size() &&
		 Algo.equal((InputIterator)this.container.beginGeneric(), 
			(InputIterator)this.container.endGeneric(),
			(InputIterator)c.beginGeneric());
	}
	return false;
    }

    /**
     * Returns the number of elements stored in the vector.
     */
    public int size() {
	return container.size();
    }

    /**
     * Returns <code>true</code> if the queue does not contain any elements.
     */
    public boolean empty() {
	return container.empty();
    }

    /**
     * Returns and removes the top element on the queue.
     */
    public void pop() {
	container.pop_front();
    }

    /**
     * Insert a new element on the top of the Queue..
     */
    public void push(Object o) {
	container.push_back(o);
    }

    /**
     * Return the first element of the Queue. Undefined if the queue is empty.
     */
    public Object front() {
	return container.front();
    }

    /**
     * Return the last element of the Queue. Undefined if the queue is empty.
     */
    public Object back() {
	return container.back();
    }
}
