package sjl;

class RBtree {
    RBtreeNode header;
    int node_count;
    boolean insert_always;
    static RBtreeNode NIL;
    Predicate2 predicate;
    Function1 keyvalue;

    public RBtree(Predicate2 pred, Function1 key, boolean insert_always) {
	init(insert_always);
	predicate = pred;
	keyvalue = key;
    }

    public RBtree(RBtree tree) {
	node_count = tree.node_count;
	predicate = tree.predicate;
        keyvalue = tree.keyvalue;
        insert_always = tree.insert_always;
	header = get_node();
	header.isRed = true;
	header.parent = __copy(tree.header.parent, header);
	if (header.parent == NIL) {
	    header.left = header;
	    header.right = header;
	} else {
	    header.left = minimum(header.parent);
	    header.right = maximum(header.parent);
	}
    }

    public RBtreeNode __insert(RBtreeNode x, RBtreeNode y, Object value) {
	++node_count;
	RBtreeNode z = get_node();
	z.value = value;
	if (y == header || x != NIL || cmpvalues(value, y.value)) {
	    y.left = z; // also makes leftmost() = z when y == header
	    if (y == header) {
		header.parent = z;
		header.right = z;
	    }
	    else if (y == header.left)
		header.left = z; // maintain leftmost() pointing to minimum
	} else {
	    y.right = z;
	    if (y == header.right)
		header.right = z; // maintain rightmost() pointing to maximum
	}
	z.parent = y;
	z.left = NIL;
	z.right = NIL;
	x = z;
	x.isRed = true;
	while (x != header.parent && x.parent.isRed) {
	    if (x.parent == x.parent.parent.left) {
		y = x.parent.parent.right;
		if (y.isRed) {
		    x.parent.isRed = false;
		    y.isRed = false;
		    x.parent.parent.isRed = true;
		    x = x.parent.parent;
		} else {
		    if (x == x.parent.right) {
			x = x.parent;
			rotate_left(x);
		    }
		    x.parent.isRed = false;
		    x.parent.parent.isRed = true;
		    rotate_right(x.parent.parent);
		}
	     } else {
		y = x.parent.parent.left;
		if (y.isRed) {
		    x.parent.isRed = false;
		    y.isRed = false;
		    x.parent.parent.isRed = true;
		    x = x.parent.parent;
		} else {
		    if (x == x.parent.left) {
			x = x.parent;
			rotate_right(x);
		    }
		    x.parent.isRed = false;
		    x.parent.parent.isRed = true;
		    rotate_left(x.parent.parent);
		}
	    }	
	}
	header.parent.isRed = false;
	return z;
    }

    public RBtreeNode __copy(RBtreeNode x, RBtreeNode p) {
	// structural copy
	RBtreeNode r = x;
	while (x != NIL) {
	    RBtreeNode y = get_node();
	    if (r == x) r = y; // save for return value
	    y.value = x.value;
	    p.left = y;
	    y.parent = p;
	    y.isRed = x.isRed;
	    y.right = __copy(x.right, y);
	    p = y;
	    x = x.left;
	}
	p.left = NIL;
	return r;
    }

    public void erase(RBtreeNode z) {
	RBtreeNode y = z;
	RBtreeNode x;
	if (y.left == NIL)
	    x = y.right;
	else
	    if (y.right == NIL)
		x = y.left;
	    else {
		y = y.right;
		while (y.left != NIL)
		    y = y.left;
		x = y.right;
	    }
	if (y != z) { // relink y in place of z
	    z.left.parent = y;
	    y.left = z.left;
	    if (y != z.right) {
		x.parent = y.parent; // possibly x == NIL
		y.parent.left = x;   // y must be a left child
		y.right = z.right;
		z.right.parent = y;
	    } else
		x.parent = y; // neaded in case X == NIL
	    if (header.parent == z)
		header.parent = y;
	    else if (z.parent.left == z)
		z.parent.left = y;
	    else
		z.parent.right = y;
	    y.parent = z.parent;
	    boolean tmp = y.isRed;
	    y.isRed = z.isRed; 
	    z.isRed = tmp;
	    y = z;
		// y points to node to be actually deleted
	} else { // y == z
	    x.parent = y.parent;  // possibly x == NIL
	    if (header.parent == z)
		header.parent = x;
	    else
		if (z.parent.left == z)
		    z.parent.left = x;
		else
		    z.parent.right = x;
	    if (header.left == z)
		if (z.right == NIL) // z.left must be NIL also
		    header.left = z.parent;
		    // makes header.left == header if z == header.parent
	        else
		     header.left = minimum(x);
	    if (header.right == z)
		if (z.left == NIL) // z.right must be NIL also
		    header.right = z.parent;
		    // makes header.right == header if z == header.parent
		else
		    header.right = maximum(x);
	}
	if (y.isRed == false) {
	    while (x != header.parent && x.isRed == false)
		if (x == x.parent.left) {
		    RBtreeNode w = x.parent.right;
		    if (w.isRed == true) {
			w.isRed = false;
			x.parent.isRed = true;
			rotate_left(x.parent);
			w = x.parent.right;
		    }
		    if (w.left.isRed == false && w.right.isRed == false) {	
			w.isRed = true;
			x = x.parent;
		    } else {
			if (w.right.isRed == false) {
			    w.left.isRed = false;
			    w.isRed = true;
			    rotate_right(w);
			    w = x.parent.right;
			}
			w.isRed = x.parent.isRed;
			x.parent.isRed = false;
			w.right.isRed = false;
			rotate_left(x.parent);
			break;
		    }
		} else { // same as then clause with "right" and "left" exchanged"
		    RBtreeNode w = x.parent.left;
		    if (w.isRed == true) {
			w.isRed = false;
			x.parent.isRed = true;
			rotate_right(x.parent);
			w = x.parent.left;
		    }
		    if (w.right.isRed == false && w.left.isRed == false) {	
			w.isRed = true;
			x = x.parent;
		    } else {
			if (w.left.isRed == false) {
			    w.right.isRed = false;
			    w.isRed = true;
			    rotate_left(w);
			    w = x.parent.left;
			}
			w.isRed = x.parent.isRed;
			x.parent.isRed = false;
			w.left.isRed = false;
			rotate_right(x.parent);
			break;
		    }
		}
	    x.isRed = false;
	}
	y.value = y.left = y.right = null;
	--node_count;
    }

    public int erase(Object value) {
	Pair p = equal_range(value);
	int n = distance((RBtreeNode)p.first, (RBtreeNode)p.second);
	erase((RBtreeNode)p.first, (RBtreeNode)p.second);
	return n;
    }

    public void erase(RBtreeNode first, RBtreeNode last) {
	if (first == begin() && last == end() && node_count != 0) {
	    header.left = header;
	    header.parent = NIL;
	    header.right = header;
	    node_count = 0;
	} else
	    while (first != last) {
		RBtreeNode next = _next(first);
		erase(first);
		first = next;
	    }
    }

    RBtreeNode get_node() {
	return new RBtreeNode();
    }

    public void init(boolean insert_always) {
	if (NIL == null) {	
	    NIL = get_node();
	    NIL.isRed = false;
	    NIL.parent = null;
	    NIL.left = null;
	    NIL.right = null;
	}
	header = get_node();
	header.isRed = true;
	header.parent = NIL;
	header.left = header;
	header.right = header;
	this.insert_always = insert_always;
    }

    public void dump() {
	System.out.println(header.toString2());
    }

    public int size() {
	return node_count;
    }

    public RBtreeNode minimum(RBtreeNode x) {
	while (x.left != NIL)
	    x = x.left;
	return x;
    }

    public RBtreeNode maximum(RBtreeNode x) {
	while (x.right != NIL)
	    x = x.right;
	return x;
    }

    public RBtreeNode lower_bound(Object value) {
	RBtreeNode y = header;  	// Last node which is not less than value
	RBtreeNode x = header.parent;	// Current node
	while (x != NIL)
	    if (!cmpvalue1(x.value, value)) {
		y = x;
		x = x.left;
	    } else
		x = x.right;
	return y;
    }

    public RBtreeNode upper_bound(Object value) {
	RBtreeNode y = header;  	// Last node which is not less than value
	RBtreeNode x = header.parent;	// Current node
	while (x != NIL)
	    if (cmpvalue2(value, x.value)) {
		y = x;
		x = x.left;
	    } else
		x = x.right;
	return y;
    }

    public Pair equal_range(Object value) {
	return new Pair(lower_bound(value), upper_bound(value));
    }

    public int count(Object value) {
	Pair p = equal_range(value);
	return distance((RBtreeNode)p.first, (RBtreeNode)p.second); 
    }

    public RBtreeNode find(Object value) {
	RBtreeNode y = header;	      // Last node which is not less than value
	RBtreeNode x = header.parent; // Current node

	while (x != NIL) 
	    if (!cmpvalue1(x.value, value)) {
		y = x;
		x = x.left;
	    } else
		x = x.right;
	return (y == end() || cmpvalue2(value, y.value)) ? end() : y;
    }

    public int distance(RBtreeNode first, RBtreeNode last) {
	int n = 0;
	while (first != last) {
	    first = _next(first);
	    n++;
	}
	return n;
    }

    public Pair insert(Object value) {
	RBtreeNode y = header;
	RBtreeNode x = header.parent;
	boolean comp = true;
	while (x != NIL) {
	    y = x;
	    comp = cmpvalues(value, x.value);
	    x = comp ? x.left : x.right;
	}
	if (insert_always)
	    return pair_iterator_bool(__insert(x, y, value), true);
	RBtreeNode j = y;
	if (comp)
	    if (j == header.left)
		return pair_iterator_bool(__insert(x, y, value), true);
	    else
		j = _prev(j);
	if (cmpvalues(j.value, value))
	    return pair_iterator_bool(__insert(x, y, value), true);
	return pair_iterator_bool(j, false);
    }

    public RBtreeNode insert(RBtreeNode position, Object value) {
	if (position == begin())
	    if (size() > 0 && cmpvalues(value, position.value))
		return __insert(position, position, value);
		// first argument just need to be non-NIL
	    else
		return (RBtreeNode)insert(value).first;
	else if (position == end())
	    if (cmpvalues(header.right.value, value))
		return __insert(NIL, header.right, value);
	    else
		return (RBtreeNode)insert(value).first;
	else {
	    RBtreeNode before = _prev(position);
	    if (cmpvalues(before.value, value)
			&& cmpvalues(value, position.value))
		if (before.right == NIL)
		    return __insert(NIL, before, value);
		else
		    return __insert(position, position, value);
		    // first argument just need to be non-NIL
	    else
		return (RBtreeNode)insert(value).first;
	}
    }

    private void rotate_left(RBtreeNode x) {
	RBtreeNode y = x.right;
	x.right = y.left;
	if (y.left != NIL)
	    y.left.parent = x;
	y.parent = x.parent;
	if (x == header.parent)
	    header.parent = y;
	else if (x == x.parent.left)
	    x.parent.left = y;
	else 
	    x.parent.right = y;
	y.left = x;
	x.parent = y;
    }

    private void rotate_right(RBtreeNode x) {
	RBtreeNode y = x.left;
	x.left = y.right;
	if (y.right != NIL)
	    y.right.parent = x;
	y.parent = x.parent;
	if (x == header.parent)
	    header.parent = y;
	else if (x == x.parent.right)
	    x.parent.right = y;
	else 
	    x.parent.left = y;
	y.right = x;
	x.parent = y;
    }

    private Pair pair_iterator_bool(RBtreeNode x, boolean bool) {
	return new Pair(x, new Boolean(bool)); // XXX:
    }

    public RBtreeNode begin() {
	return header.left;
    }

    public RBtreeNode end() {
	return header;
    }

    public RBtreeNode _next(RBtreeNode node) {
//	System.out.println(node.toString2());
	if (node.right != NIL) {
	    node = node.right;
	    while (node.left != NIL)
		node = node.left;
	} else {
	    RBtreeNode y = node.parent;
	    while (node == y.right) {
		node = y;
		y = y.parent;
	    }
	    if (node.right != y) // necessary because of rightmost
		node = y;
	}
	return node;
    }

    public RBtreeNode _prev(RBtreeNode node) {	
	if (node.isRed && node.parent.parent == node) // check for header
	    node = node.right; // return rightmost
	else if (node.left != NIL) {
	    RBtreeNode y = node.left; 
	    while (y.right != NIL)
		y = y.right;
	    node = y;
	} else {
	    RBtreeNode y = node.parent;
	    while (node == y.left) {
		node = y;
		y = y.parent;
	    }
	    node = y;
	}
	return node;
    }

    private boolean cmpvalues(Object x, Object y) {
	// System.out.println("cmpvalues " + x + " " + y);
  	return predicate.compare(keyvalue.perform(x), keyvalue.perform(y));
    }
    private boolean cmpvalue1(Object x, Object y) {
	// System.out.println("cmpvalue1 " + x + " " + y);
  	return predicate.compare(keyvalue.perform(x), y);
    }
    private boolean cmpvalue2(Object x, Object y) {
	// System.out.println("cmpvalue2 " + x + " " + y);
  	return predicate.compare(x, keyvalue.perform(y));
    }
}
