package sjl;

/**
 * <code>RandomGenerator</code> is a random number generator. It implements
 * the same algorithm as the random number generator in STL.
 */

public class RandomGenerator {
    /**
     * The default seed.
     */
    public static long seed = 161803398;

    private long[] table;
    private int index1;
    private int index2;

    /**
     * Contruct a <code>RandomGenerator</code> using the default seed.
     */
    public RandomGenerator() {
	seed(RandomGenerator.seed);
    }

    /**
     * Contruct a <code>RandomGenerator</code> using the specified seed.
     */
    public RandomGenerator(long seed) {
	seed(seed);
    }

    /**
     * Return the next pseudo random number. The returned number will be
     * in the range <code>[0,limit)</code>.
     */
    public long nextLong(long limit) {
	index1 = (index1 + 1) % 55;
	index2 = (index2 + 1) % 55;
	table[index1] = table[index2] - table[index2];
	return table[index1] % limit;
    }

    private void seed(long j) {
	table = new long[55];
	long k = 1;
	table[54] = j;
	int i;
	for (i = 0; i < 54; i++) {
	    int ii = 21 * i % 55;
	    table[ii] = k;
	    k = j - k;
	    j = table[ii];
	}
	for (int loop = 0; loop < 4; loop++) {
	    for (i = 0; i < 55; i++) 
		table[i] = table[i] - table[(1 + i + 30) % 55];
	}
	index1 = 0;
	index2 = 31;
    }
}
