package sjl;

/**
 * Defines the methods allowed on random iterators. Random iterators allows 
 * long-jumping in both directions, and comparing two iterators for 
 * relativ ordering.
 *
 * <p>Copyright <a href=..\Copyrght.htm>&copy</a> 1996 Finn Bock<p>
 */
public interface RandomIterator extends BidirectionalIterator {

    /**
     * Return true if the position of this is lower than the position of target
     * @param target the iterator to compare with.
     * @return true if this is lower than target.
     */
    abstract boolean less(Iterator target);

    /**
     * Return the object placed distance elements away.
     * @param distance a relative offset.
     * @return Return the object placed distance elements away.
     */
    public abstract Object get(int distance);

    /**
     * Place the object in the position distance elements away.
     * @param object the object to put into the container.
     * @param distance a relative offset.
     * @return The object just inserted.
     */
    public abstract Object put(int distance, Object object);

    /**
     * Advance the iterator distance elements backards (or forwards if 
     * distance is negative).
     * @param distance a relative offset.
     * @return this.
     */
    public abstract RandomIterator prev(int distance);

    /**
     * Advance the iterator distance elements forwards (or backwards if 
     * distance is negative).
     * @param distance a relative offset.
     * @return this.
     */
    public abstract RandomIterator next(int distance);
}
