package sjl;

/**
 * Defines the methods allowed on sequence container types. 
 * The supported containers are currently:
 * <code>List</code>, <code>Vector</code> and <code>Deque</code>.
 *
 * <p>Copyright <a href=..\Copyrght.htm>&copy</a> 1996 Finn Bock<p>
 *
 * @see List
 * @see Vector
 * @see Deque
 */
public interface SequenceContainer extends Container {

    /**
     * Insert a copy of object before position.
     */
    public abstract Iterator insert(Iterator position, Object object);

    /**
     * Insert <code>n</code> copies of <code>object</code> before position.
     */
    public abstract void insert(Iterator position, int n, Object object);

    /**
     * Insert copies of <code>[first, last)</code> before position.
     */
    public abstract void insert(Iterator position, InputIterator first, InputIterator last);

    /**
     * Erase the element pointed to by position.
     */
    public abstract void erase(Iterator position);

    /**
     * Erase the elements in the range <code>[first,last)</code>.
     */
    public abstract void erase(Iterator first, Iterator last);

    /**
     * Return the first element in the container. Undefined is the container
     * is empty.
     */
    public abstract Object front();

    /**
     * Return the last element in the container. Undefined is the container
     * is empty.
     */
    public abstract Object back();
}
