package sjl;

/**
 * <code>Stack</code> is a container adapter that provides a 
 * <i>last-in, first-out</i> protocol to the elements in the 
 * container. The container must implement the 
 * <a href=sjl.BackInsertContainer.html>BackInsertContainer</a> interface.
 * The <code>Vector</code>, <code>List</code> and <code>Deque</code> 
 * containers implements the <code>BackInsertContainer</code> interface.
 *
 * <p>Copyright <a href=..\Copyrght.htm>&copy</a> 1996 Finn Bock<p>
 * 
 * @see Vector
 * @see Deque
 * @see List
 */ 

public class Stack {
    BackInsertContainer container;

    /**
     * Construct a new Stack based on the container argument.
     */
    public Stack(BackInsertContainer container) {
	this.container = container;
    }

    /**
     * Compare the elements in this container with the elements 
     * in another container.
     * @return <code>true</code> is the elements match.
     */
    public boolean equals(Object container) {
	if ((container != null) && (container instanceof Container)) {
	    Container c = (Container)container;
	    return this.container.size() == c.size() &&
		 Algo.equal((InputIterator)this.container.beginGeneric(), 
			(InputIterator)this.container.endGeneric(),
			(InputIterator)c.beginGeneric());
	}
	return false;
    }

    /**
     * Returns the number of elements stored in the vector.
     */
    public int size() {
	return container.size();
    }

    /**
     * Returns <code>true</code> if the stack does not contain any elements.
     */
    public boolean empty() {
	return container.empty();
    }

    /**
     * Returns and removes the top element on the stack.
     */
    public void pop() {
	container.pop_back();
    }

    /**
     * Insert a new element on the top of the Stack..
     */
    public void push(Object o) {
	container.push_back(o);
    }

    /**
     * Return the top element of the Stack. Undefined if the stack is empty.
     */
    public Object top() {
	return container.back();
    }
}
